Instance

Struct Instance 

pub struct Instance<'a, Tag> { /* private fields */ }
An imported FMI 3.0 instance

Implementations

Source

impl<'a> Instance<'a, CS>

pub fn new( import: &'a Fmi3Import, instance_name: &str, visible: bool, logging_on: bool, event_mode_used: bool, early_return_allowed: bool, required_intermediate_variables: &[fmi3ValueReference], ) -> Result<Self, Error>

Returns a new CoSimulation instance.

An FMU can be instantiated many times (provided capability flag can_be_instantiated_only_once_per_process = false).

Arguments:

  • instance_name: a unique identifier for the FMU instance. It is used to name the instance, for example, in error or information messages generated by one of the fmi3XXX functions. The argument instance_name must be a non empty string (in other words, must have at least one character that is not a white space). If only one FMU is simulated, either the attribute crate::fmi3::schema::Fmi3ModelDescription::model_name or one of

  • visible: Defines that the interaction with the user should be reduced to a minimum (no application window, no plotting, no animation, etc.). In other words, the FMU is executed in batch mode. If visible = true, the FMU is executed in interactive mode, and the FMU might require to explicitly acknowledge start of simulation / instantiation / initialization (acknowledgment is non-blocking).

  • logging_on: If = false, then any logging is disabled and the logMessage callback function must not be called by the FMU. If logging_on = true, then all LogCategories are enabled. The function crate::fmi3::traits::Common::set_debug_logging gives more detailed control about enabling specific LogCategories.

  • event_mode_used: If = true the importer can handle events. The flag may only be true, if has_event_mode = true, otherwise the FMU must raise an error. For FMUs that have clocks, event_mode_used = true is required.

  • early_return_allowed: If = true the importer can handle early return. Only in this case, crate::fmi3::traits::CoSimulation::do_step() may return with early_return = true.

  • required_intermediate_variables: An array of the value references of all input variables that the simulation algorithm intends to set and all output variables it intends to get during intermediate updates. This set may be empty (nRequiredIntermediateVariables = 0) when the simulation algorithm does not intend to use intermediate update. Only the variables in requiredIntermediateVariables may be accessed by the simulation algorithm using fmi3Set{VariableType} and fmi3Get{VariableType} during Intermediate Update Mode. All variables referenced in this set must be marked with the attribute intermediateUpdate = “true” in modelDescription.xml.

See: https://fmi-standard.org/docs/3.0.1/#fmi3InstantiateCoSimulation

Source

impl<'a> Instance<'a, ME>

pub fn new( import: &'a Fmi3Import, instance_name: &str, visible: bool, logging_on: bool, ) -> Result<Self, Error>

Source

impl<'a> Instance<'a, SE>

pub fn new( import: &'a Fmi3Import, instance_name: &str, visible: bool, logging_on: bool, ) -> Result<Self, Error>

Source

impl<'a, Tag> Instance<'a, Tag>
where Self: Common,

pub fn get_variable_dimensions(&mut self, var_refs: &[u32]) -> usize

Get the sum of the product of the dimensions of the variables with the given value references.