#[repr(C)]pub enum StatusKind {
DoStepStatus = 0,
PendingStatus = 1,
LastSuccessfulTime = 2,
Terminated = 3,
}Available on crate feature
fmi2 only.Variants
DoStepStatus = 0
Can be called when the fmi2DoStep function returned fmi2Pending. The function delivers fmi2Pending if the computation is not finished. Otherwise the function returns the result of the asynchronously executed fmi2DoStep call
PendingStatus = 1
Can be called when the fmi2DoStep function returned fmi2Pending. The function delivers a string which informs about the status of the currently running asynchronous fmi2DoStep computation.
LastSuccessfulTime = 2
Returns the end time of the last successfully completed communication step. Can be called after fmi2DoStep(…) returned fmi2Discard.
Terminated = 3
Returns true, if the slave wants to terminate the simulation. Can be called after fmi2DoStep(…) returned fmi2Discard. Use fmi2LastSuccessfulTime to determine the time instant at which the slave terminated